/*
 +---------------------------------------------------------------------+
 | apcsmart.h                                                          |
 | header file for APC Smart UPS 700 device                            |
 |                                                                     |
 | Author:  Bob Pill, NCIS                                             |
 |          bobp@newcorridor.com                                       |
 |                                                                     |
 | Created: Sept 15, 1997                                              |
 |                                                                     |
 | Copyright (c) 1997 New Corridor Information Services.               |
 | All rights reserved.                                                |
 |                                                                     |
 |                                                                     |
 | THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR        |
 | IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED      |
 | WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR          |
 | PURPOSE.                                                            |
 |                                                                     |
 |---------------------------------------------------------------------|
 |  ***Revision-History***  Begin-Log                                  |
 |     Initial creation - (started at ver 2.0.1) Sept 15, 1997 (BP)    |
 |     Version 2.1.0 update Jan 01, 1998 (BP)                          |
 |     Version 2.1.1 update Feb 03, 1998 (BP)                          |
 |     Version 2.1.2 update May 17, 1998 (BP)                          |
 |     Version 2.1.3 update Aug 04, 1998 (BP)                          |
 |  ***Revision-History***  End-Log                                    |
 +---------------------------------------------------------------------+
*/


#ifndef __APCSMART_H

#define __APCSMART_H


/*
 +---------------------------------------------------------------------+
 |   Smart UPS commands                                                |
 |                                                                     |
 | The first description for this has been given by Ty Hoeffer,        |
 | <pth3k@galen.med.virginia.edu>.                                     |
 |                                                                     |
 | This more complete description given later by Joerg Wunsch          |
 | <joerg_wunsch@interface-business.de> (Oct 30, 1995)                 |
 |                                                                     |
 | Procedure for Smart UPS:                                            |
 |                                                                     |
 | Simple three-wire V.24, 2400 Bd, 8 bits/char, no parity.  Smart     |
 | mode must be turned on (UPS_CMD_SMART).                             |
 | The UPS issues single-byte event codes for some conditions.         |
 | The controller can issue single-byte command codes.  Responses to   |
 | the command codes are multi-character strings, terminated by \r\n.  |
 | (In addition, the test commands cause the UPS to generate a         |
 | powerfail event.)                                                   |
 |                                                                     |
 | The commands to turn the UPS on or off must be issued twice, with   |
 | at least one second pause between.                                  |
 |                                                                     |
 | The way to reprogram variable settings is not entirely clear.       |
 | Numerical variables can be altered by issuing the command, and then |
 | UPS_CMD_INCR, or UPS_CMD_DECR, respectively.  The UPS will respond  |
 | with the new register setting, in its internal form (00, 01, 02, or |
 | 03).  For string values, the procedure is to specifiy the variable  |
 | to modify, then either UPS_CMD_INCR, or UPS_CMD_DECR, followed by   |
 | the exact number of bytes for this setting (with moderate delays    |
 | between the characters, remember there's no flow control).  The UPS |
 | will respond with OK finally.                                       |
 |                                                                     |
 | Probably one needs to issue the command '1' (twice, with one second |
 | pause) in order to store the changed data in NVRAM?  (The UPS is    |
 | responding with PROG to this command.)                              |
 |                                                                     |
 | Commands denoted as UPS_CMD_XXX* are unknown, but produce valid     |
 | result strings.                                                     |
 +---------------------------------------------------------------------+
 */

// Note: commands commented with !! return values of programmable
// registers.

#define UPS_CMD_INCR  '+'           // increment numerical setting
#define UPS_CMD_DECR  '-'           // decrement numerical setting

#define UPS_CMD_MODEL  ('a'&0x1F)   // get the model string
#define UPS_CMD_TURNON ('n'&0x1F)   // turn on UPS, must be sent twice
#define UPS_CMD_XXX1   ('z'&0x1F)   // yields tons of garbage, can't make
                                    // sense out of it 
#define UPS_CMD_PANEL   'A'         // test front panel, yields OK
#define UPS_CMD_BATTERY 'B'         // battery voltage, ##.#
#define UPS_CMD_XXX2    'C'         // some number (temperature?), ###.#
#define UPS_CMD_XXX3    'D'         // toggle line/battery operation;
                                    // doesn't always work, sometimes
                                    // only being responded with "NO"
#define UPS_CMD_AUTOTEST 'E'        // !! automatic self-test, four options
                                    // 336 (default), after 14 days
                                    // 168 after 7 days
                                    // ON  at power-on
                                    // OFF never
#define UPS_CMD_FREQ        'F'     // line frequency, Hz
#define UPS_CMD_XXX4        'G'     // R, S or T
#define UPS_CMD_SHUT_GRACE  'K'     // shutdown with grace period, issue twice
#define UPS_CMD_SHUT_SOFT   'S'     // soft shutdown - UPS will fire back up when power is restored
#define UPS_CMD_LINE        'L'     // input line voltage ###.#
#define UPS_CMD_MINLINE     'M'     // minimal line voltage rcvd
#define UPS_CMD_MAXLINE     'N'     // maximal line voltage rcvd
#define UPS_CMD_OUTPUT      'O'     // output voltage
#define UPS_CMD_POWER       'P'     // power load, in % ##.#
#define UPS_CMD_STATUS      'Q'     // status flags, two hex digits:
 #define UPS_STATUS_STBY    4       // standby, no line power
 #define UPS_STATUS_ON      8       // on, line power available
 #define UPS_STATUS_PWRF 0x10       // on, no line power
 #define UPS_STATUS_BLOW 0x40       // battery low
                                    // status == 0: off, line power available
#define UPS_CMD_DUMB        'R'     // turn into dumb mode, expect BYE
#define UPS_CMD_SELFT1      'U'     // self test
#define UPS_CMD_VERS        'V'     // mystic version string
#define UPS_CMD_SELFT2      'W'     // self test
#define UPS_CMD_XXX5        'X'     // OK, or NO
#define UPS_CMD_SMART       'Y'     // turn on smart mode, expect SM
#define UPS_CMD_SHUT_IMM    'Z'     // shutdown immediately, issue twice
#define UPS_CMD_SHOW_CMDS   'a'     // show all valid events and cmds
#define UPS_CMD_FIRMWARE    'b'     // !! firmware revision
                                    // (several weird values selectable)
#define UPS_CMD_LOCALID     'c'     // !! UPS_CMD_IDEN
#define UPS_CMD_RETTHRESH   'e'     // !! 00/01/02/03 = 00, 15, 50, 90
                                    // % battery charge threshold for
                                    // return from shutdown
#define UPS_CMD_LEVEL       'f'     // battery fill level, in % ##.#
#define UPS_CMD_NBATTERY    'g'     // nominal battery voltage, ###
#define UPS_CMD_XXX6        'j'     // yields ####:
#define UPS_CMD_ALDELAY     'k'     // !! 00/01/02/03 = 0, T, L, N
                                    // 0 = 5 second delay after line fail
                                    // T = 30 seconds delay,
                                    // L = alarm at low battery only
                                    // N = no alarm at all
#define UPS_CMD_LOWXFR      'l'     // !! 00/01/02/03 = 196,188,208,204 V
                                    // low transfer voltage
#define UPS_CMD_MDATE       'm'     // !! manufacturing date MM/DD/YY
#define UPS_CMD_ID          'n'     // !! serial #, 14 chars
#define UPS_CMD_NLINE       'o'     // !! 00/01/02/03 = 230,240,220,225 V
                                    // output voltage
#define UPS_CMD_SHUTDELAY   'p'     // !! 00/01/02/03 = 020,180,300,600
                                    // UPS_CMD_SHUT_GRACE delay, seconds
#define UPS_CMD_LOWB_WARN   'q'     // !! 00/01/02/03 = 2,5,7,10 minutes
                                    // low battery warning time, minutes
#define UPS_CMD_SHUTRETURN  'r'     // !! 00/01/02/03 = 000,060,180,300
                                    // return from shutdown delay, seconds
#define UPS_CMD_SENSITIVITY 's'     // !! 00/01/02/03 = H, M, L, or L
#define UPS_CMD_UPXFR       'u'     // !! 00/01/02/03 = 253,264,271,280 V
                                    // upper transfer voltage
#define UPS_CMD_REPLDATE    'x'     // !! date last battery replacement MM/DD/YY
#define UPS_CMD_COPYRIGHT   'y'     // Copyright string
#define UPS_CMD_XXX7        'z'     // CLEAR

// There are quite a few more commands; look at the output of SHOW_CMDS

// A few result strings

#define UPS_RES_OK    "OK"          // success
#define UPS_RES_NA    "NA"          // not applicable
#define UPS_RES_SM    "SM"          // smart mode on
#define UPS_RES_BYE   "BYE"         // smart mode off
#define UPS_RES_NO    "NO"          // attempt to modify fixed value
#define UPS_RES_PROG  "PROG"        // programmed ok (?)

// Event characters

#define UPS_EV_POWER    '!'         // input line power fail, sent periodically
#define UPS_EV_PWRBACK  '$'         // input line power back
#define UPS_EV_BATLOW   '%'         // battery low, one-time (also during test)
#define UPS_EV_BATUP    '+'         // battery up again
#define UPS_EV_TURNON   '?'         // UPS turned on; respond with UPS_SMART


#endif  // __APCSMART_H


//---------------------------------------------------------------------
// end-of-file: apcsmart.h                                             
//---------------------------------------------------------------------
