/*
 +---------------------------------------------------------------------+
 | main.cc                                                             |
 | main module for upsmond - APC Smart UPS 700 monitor                 |
 |                                                                     |
 | Author:  Bob Pill, NCIS                                             |
 |          bobp@newcorridor.com                                       |
 |                                                                     |
 | Created: Sept 15, 1997                                              |
 |                                                                     |
 | Copyright (c) 1997 New Corridor Information Services.               |
 | All rights reserved.                                                |
 |                                                                     |
 |                                                                     |
 | THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR        |
 | IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED      |
 | WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR          |
 | PURPOSE.                                                            |
 |                                                                     |
 |---------------------------------------------------------------------|
 |  ***Revision-History***  Begin-Log                                  |
 |     Initial creation - (started at ver 2.0.1) Sept 15, 1997 (BP)    |
 |     Version 2.1.0 update Jan 01, 1998 (BP)                          |
 |     Version 2.1.1 update Feb 03, 1998 (BP)                          |
 |     Version 2.1.2 update May 17, 1998 (BP)                          |
 |     Version 2.1.3 update Aug 04, 1998 (BP)                          |
 |  ***Revision-History***  End-Log                                    |
 +---------------------------------------------------------------------+
*/

#include <sys/types.h>
#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>
#include <string.h>
#include <memory.h>
#include <syslog.h>

#include "stddef.h"
#include "daemon.h"
#include "upsmonitor.h"

// upsmond version number
static char g_szUpsmondVersion[] = "2.1.3";

// UPS monitor object
static UpsMonitor* g_pMonitor = NULL;   

// shutdown script invoked to shutdown system
static char *g_aszScript[] = { "/sbin/shutdown", "-h", "now", "Power Failure : system shutdown", 0 };

// defaults - change defaults to conform to system as needed
static char g_szDefaultPIDFilePath[] = "/var/run/upsmond.pid";
static char g_szDefaultPortName[] = "/dev/tty00";
static char g_szDefaultSocketName[] = "/tmp/upsmond.sock";
static char g_szDefaultTempDirectory[] = "/tmp";
static int  g_iSyslogFacility = LOG_DAEMON;

// syslog facility table
typedef struct syslog_facility
{
  char *pszFacility;
  int iFacility;
} SYSLOG_FACILITY;

static SYSLOG_FACILITY slfTable[] =
{
  {"LOG_DAEMON",LOG_DAEMON},
  {"LOG_LOCAL0",LOG_LOCAL0},
  {"LOG_LOCAL1",LOG_LOCAL1},
  {"LOG_LOCAL2",LOG_LOCAL2},
  {"LOG_LOCAL3",LOG_LOCAL3},
  {"LOG_LOCAL4",LOG_LOCAL4},
  {"LOG_LOCAL5",LOG_LOCAL5},
  {"LOG_LOCAL6",LOG_LOCAL6},
  {"LOG_LOCAL7",LOG_LOCAL7},
  {NULL,0},
};


void Usage(char *pszProgramName)
{
    printf("\nusage: %s [-hilpsrtd]\n",pszProgramName);
    printf("        [-h help] (display this help)\n");
    printf("        [-i PID file name] (default is %s)\n",g_szDefaultPIDFilePath);
    printf("        [-l syslog facility] (default is LOG_DAEMON)\n");
    printf("        [-p port name] (default is %s)\n",g_szDefaultPortName);
    printf("        [-r retry limit] (default is %d minutes)\n",DEFAULT_RETRY_LIMIT);
    printf("        [-s socket name] (default is %s)\n",g_szDefaultSocketName);
    printf("        [-t temp directory] (default is %s)\n",g_szDefaultTempDirectory);
    printf("        [-d : toggle debug] (default is OFF)\n\n");
}


int PostPID(const char* pszPIDFilePath)
{
  int iRetCode = RC_SUCCESS;
  FILE *fsPID = NULL;  // file stream for PID output

  fsPID = fopen(pszPIDFilePath,"w");
  if(fsPID)
  {
    fprintf(fsPID,"%d",getpid());
    fclose(fsPID);
  }
  else
  {
    printf("\nCould not open PID file %s\n",pszPIDFilePath);
    iRetCode = RC_FILE_OPEN_ERROR;
  }
  return(iRetCode);
}


void Cleanup(void)
{
    if(g_pMonitor)
    {
      delete g_pMonitor;
      g_pMonitor = NULL;
    }
    closelog();
}


void quit(int iExitMode)
{

    switch(iExitMode)
    {
      case UPSMONITOR_EXIT_SHUTDOWN:
        // tell UPS to shutdown, will give us 30 secs to shutdown here
        g_pMonitor->ShutdownUPS();
        Cleanup();

        /* execute O/S shutdown */  
        execvp(g_aszScript[0], g_aszScript);

        /* if exec fails, then log message and exit */
        syslog(LOG_ALERT,"Unable to shutdown system.  errno: %d",errno);
        exit(0);
        break;

      case UPSMONITOR_EXIT_NORMAL:
      case SIGINT:
      case SIGHUP:
      case SIGTERM:
      default:
        syslog(LOG_INFO,"Exiting upsmond daemon.");
        Cleanup();
        exit(0);
        break;
    }
}


int main(int argc, char **argv)
{
    int iRetCode = RC_SUCCESS;
    BOOL bDebug = FALSE;
    int iRetryLimit = DEFAULT_RETRY_LIMIT;
    SYSLOG_FACILITY *pFacility = NULL;
    int iErrors = 0;
    int iExitMode;
    int iOption;
    char *pszSlash;
    char szProgramName[MAX_PATHNAME];
    char szPIDFilePath[MAX_PATHNAME];
    char szPortName[MAX_PATHNAME];
    char szSocketName[MAX_PATHNAME];
    char szLogFacility[MAX_PATHNAME];
    char szTempDirectory[MAX_PATHNAME];


    // initialization
    memset(szProgramName,0,sizeof(szProgramName));
    memset(szPIDFilePath,0,sizeof(szPIDFilePath));
    memset(szPortName,0,sizeof(szPortName));
    memset(szSocketName,0,sizeof(szSocketName));
    memset(szLogFacility,0,sizeof(szLogFacility));
    memset(szTempDirectory,0,sizeof(szTempDirectory));
    strcpy(szPortName,g_szDefaultPortName);
    strcpy(szPIDFilePath,g_szDefaultPIDFilePath);
    strcpy(szSocketName,g_szDefaultSocketName);
    strcpy(szTempDirectory,g_szDefaultTempDirectory);
    if((pszSlash = strrchr(argv[0], '/')))
      strcpy(szProgramName,pszSlash+1);
    else
      strcpy(szProgramName,argv[0]);
  
    // parse command line options
    while((iOption = getopt(argc, argv, "dhi:l:p:r:s:t:")) != EOF)
    {
      switch(iOption)
      {
        case 'd':
          bDebug = TRUE;
          printf("Debug option set\n");
          break;
        
        case 'h':
          Usage(szProgramName);
          goto MainExit;
          break;

        case 'i':
          strcpy(szPIDFilePath,optarg);
          printf("PID filepath set to: %s\n",szPIDFilePath);
          break;

        case 'l':
          strcpy(szLogFacility,optarg);
          for(pFacility = slfTable;pFacility->pszFacility;pFacility++)
          {
            if(strcasecmp(szLogFacility,pFacility->pszFacility) == 0)
            {
              g_iSyslogFacility = pFacility->iFacility;
              break;
            }
          }
          if(pFacility->pszFacility == NULL)
            printf("Unsupported Log Facility <%s> - using LOG_DAEMON default\n",szLogFacility);
          break;  

        case 'p':
          strcpy(szPortName,optarg);
          printf("PortName set to: %s\n",szPortName);
          break;

        case 'r':
          iRetryLimit = atol(optarg);
          if(iRetryLimit <= 0 || iRetryLimit > 30)
          {
            printf("Invalid retry limit %d ignored\n",iRetryLimit);
            iRetryLimit = DEFAULT_RETRY_LIMIT;
          }
          break;

        case 's':
          strcpy(szSocketName,optarg);
          printf("SocketName set to: %s\n",szSocketName);
          break;

        case 't':
          strcpy(szTempDirectory,optarg);
          printf("TempDirectory set to: %s\n",szTempDirectory);
          break;

        case '?':
          iErrors++;
      }
    }

    // display usage if any command line errors and exit
    if(iErrors)
    {
      Usage(szProgramName);
      iRetCode = RC_INVALID_CMDLINE_ARGS;
      goto MainExit;
    }

    // become a daemon if not in debug mode
    if(bDebug == FALSE)
    {
      iRetCode = DaemonInit(szTempDirectory);
      if(iRetCode != RC_SUCCESS)
      {
        printf("Could not initialize daemon process. %d\n",iRetCode);
        goto MainExit;
      }
    }

    // Instantiate UpsMonitor object
    g_pMonitor = new UpsMonitor(szPortName, szSocketName, iRetryLimit, g_szUpsmondVersion);
    if(g_pMonitor)
    {
      iRetCode = g_pMonitor->Init();
      if(iRetCode != RC_SUCCESS)
      {
        printf("Could not establish communication with the UPS device. %d\n",iRetCode);
        goto MainExit;
      }
    }
    else
    {
      iRetCode = RC_MEMORY_ALLOCATION_ERROR;
      goto MainExit;
    }

    // setup PID file
    iRetCode = PostPID(szPIDFilePath);
    if(iRetCode != RC_SUCCESS)
      goto MainExit;

    // install signal handlers
    signal(SIGINT, quit);
    signal(SIGHUP, quit);
    signal(SIGTERM, quit);

    // setup system logging
    openlog("UPS", LOG_NOWAIT, g_iSyslogFacility);
    syslog(LOG_INFO,"upsmond daemon started - version %s",g_szUpsmondVersion);

    // run monitor
    iExitMode = g_pMonitor->Run();
    quit(iExitMode);

MainExit:

    return iRetCode;
}


//---------------------------------------------------------------------
// end-of-file: main.cc                                                
//---------------------------------------------------------------------
